
module Talk where

import Control.Applicative
import Tomorrow

-- Streams ----------------------------------------------------------------

data Stream a = a :> O (Stream a)

nats :: Stream Integer
nats = fix $ \nats -> 0 :> (fmap (+1) <$> nats)

instance Functor Stream where
  fmap f = fix $ \fmapf (x :> xs) ->
    f x :> (fmapf <*> xs)
    -- fmapf :: O (Stream a -> Stream b)

-- Repmin -----------------------------------------------------------------

data Tree = Leaf Int | Node Tree Tree
  deriving Show

repMin :: Tree -> O Tree
repMin t = t'
  where (_, t') = fix $ \mt -> repAux (fst <$> mt, t)

repAux :: (O Int, Tree) -> (Int, O Tree)
repAux (m, Leaf i)   = (i, Leaf <$> m)
repAux (m, Node l r) = (min i j, Node <$> l' <*> r')
  where
    (i, l') = repAux (m, l)
    (j, r') = repAux (m, r)

t1 = Leaf 4 `Node` (Leaf 2 `Node` Leaf 1)

-- Spiral -----------------------------------------------------------------

spiral :: (Stream a -> (b, Stream a)) -> a -> b
spiral f a = b
  where
    (b, _) = fix $ \bas -> f (a :> (snd <$> bas))

-- Boring stuff -----------------------------------------------------------

infixr 5 :>

instance Show a => Show (Stream a) where
  show xs = show' 5 xs
    where
      show' 0 _ = "..."
      show' n (x :> xs) = showsPrec 6 x "" ++ " :> " ++
                          showsPrec 5 (fmap (Text . show' (n - 1)) xs) ""

newtype Text = Text { unText :: String }

instance Show Text where
  showsPrec p (Text s) = showParen (p>5) (showString s)

